package com.yizhi.message.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.message.application.constans.Constans;
import com.yizhi.message.application.controller.util.MessageUtil;
import com.yizhi.message.application.controller.util.QueryAccount;
import com.yizhi.message.application.domain.*;
import com.yizhi.message.application.enums.MessageStatus;
import com.yizhi.message.application.service.MessageAccountService;
import com.yizhi.message.application.service.MessageParamRelationService;
import com.yizhi.message.application.service.MessageRemindService;
import com.yizhi.message.application.service.MessageVisibleRangeService;
import com.yizhi.message.application.util.DateConvertor;
import com.yizhi.message.application.vo.MessageVo;
import com.yizhi.message.application.vo.TaskVo;
import com.yizhi.message.application.mapper.*;
import com.yizhi.system.application.vo.AccountVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author hutao123
 * @since 2019-09-09
 */
@Service
public class MessageRemindServiceImple extends ServiceImpl<MessageRemindMapper, MessageRemind> implements MessageRemindService {

    @Autowired
    private MessageRemindMapper messageRemindMapper;
    @Autowired
    private MessageVisibleRangeMapper messageVisibleRangeMapper;

    @Autowired
    private MessageMapper messageMapper;

    @Autowired
    private QueryAccount queryAccount;

    @Autowired
    private MessageAccountMapper messageAccountMapper;

    @Autowired
    private MessageAccountService messageAccountService;

    @Autowired
    private IdGenerator idGenerator;

    @Autowired
    private MessageUtil messageUtil;

    @Autowired
    private MessageContextMapper messageContextMapper;

    @Autowired
    private MessageVisibleRangeService messageVisibleRangeService;

    @Autowired
    private MessageParamRelationService messageParamRelationService;

    private static SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm");
    private static final Logger logger = LoggerFactory.getLogger(MessageRemindServiceImple.class);


    @Override
//    @Transactional(rollbackFor = Exception.class)
    public void sendMessageHandle() {

        List<MessageRemind> messageReminds = messageRemindMapper.getListByTime();
        List<AccountVO> accountVOS = null;


        if (CollectionUtils.isNotEmpty(messageReminds)) {
            for (MessageRemind remind : messageReminds) {
                logger.info("-----------messageReming:-------------" + remind.toString());
                if (remind.getMessageType() != 1) {
                    //非自定义消息需要判断业务状态，
                    Boolean up = null;
                    try {
                        up = messageUtil.isUp(remind.getRelationId(), remind.getRelationType());
                    } catch (Exception e) {
                        e.printStackTrace();
                        logger.info("判断业务是否为上架状态时，出现异常！！");
                        continue;
                    }
                    if (!up) {
                        logger.info("业务为非上架状态，不允许发消息！！！");
                    }
                }
                Message message = messageMapper.selectById(remind.getMessageId());
                if (message != null) {
                    try {
                        if (!MessageStatus.DOWN.getCode().equals(message.getStatus())) {

                            updateMessageRemindStatus(remind.getId(), MessageStatus.SEND_ING.getCode());
                            logger.info("更新待发消息的状态为发送中");
                            Integer num = 0;
                            //如果有accountId,则不用管可见范围类型，直接发
                            if (remind.getAccountId() != null) {
                                //有用户id直接发消息；
                                accountVOS = queryAccount.getAccountByIds(Arrays.asList(remind.getAccountId()));
                            } else {
                                //修改逻辑为 若为全站点发消息，则只插入一条数据，用户参数则在查询的时候替换
                                if (Constans.PUBLIC_SITE.equals(remind.getVisibleRange())) {
                                    num = insertMessageAccountByPublic(remind);
                                } else {
                                    //获取指定范围内的用户
                                    accountVOS = getAllAccounts(remind);
                                }
                            }
                            //开始发消息
                            if (CollectionUtils.isNotEmpty(accountVOS)) {
                                logger.info("获取到用户数为：" + accountVOS.size());
                                num = insertMessageAccount(accountVOS, remind);
                            } else {
                                if (remind.getAccountId() != null || (remind.getAccountId() == null && !Constans.PUBLIC_SITE.equals(remind.getVisibleRange()))) {
                                    logger.info("用户查询为空！！！");
                                }
                            }
                            if (num > 0) {
                                if (remind.getMessageType() == 1) {
                                    logger.info("更新主体消息的状态为已发送");
                                    //更新消息的状态为已发送
                                    updateMessageStatus(remind.getMessageId(), MessageStatus.SEND_ED.getCode());
                                }
                                logger.info("更新待发消息的状态为已发送");
                                //更新待发消息的状态为已发送
                                updateMessageRemindStatus(remind.getId(), MessageStatus.SEND_ED.getCode());
                            }
                            logger.info("处理完毕！！！！");
                        } else {
                            logger.info("下架状态不执行！！！");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        throw e;
                    }
                } else {
                    logger.info("数据库没有该主体消息");
                }
            }
        } else {
            logger.info("数据库没有该待发消息");
        }

    }

    /**
     * 解析参数，获取只含用户名称参数的消息内容
     *
     * @param messageId
     * @param taskVo
     * @return
     */
    public String getContext(com.yizhi.message.application.vo.MessageRemindVo vo, TaskVo taskVo) {

        String messageContext = "";
        Long siteId = vo.getRequestContext().getSiteId();
        if (vo.getMessageId() == null) {
            logger.info("查询消息内容时，id为空！");
            return messageContext;
        }
        if (null != vo.getMessageType() && vo.getMessageType() == 3) {
            if (siteId == null) {
                logger.info("siteId为空！！");
                return messageContext;
            }
            Message message = new Message();
            message.setEvenType(vo.getMessageId());
            message.setSiteId(siteId);
            message.setDeleted(Constans.NOT_DELETED);
            message = messageMapper.selectOne(message);
            if (message != null) {
                vo.setMessageId(message.getId());
                logger.info("接收到触发消息：messageId||" + message.getId());
            }
        }

        MessageContext context = new MessageContext();
        context.setMessageId(vo.getMessageId());
        context = messageContextMapper.selectOne(context);

        if (context != null) {
            logger.info("数据库查询出来的消息内容伪：" + JSON.toJSONString(context));
            if (!StringUtils.isBlank(context.getContext())) {
                messageContext = context.getContext();

                MessageParamRelation paramRelation = new MessageParamRelation();
                paramRelation.setMessageId(vo.getMessageId());
                paramRelation.setDeleted(Constans.NOT_DELETED);
                EntityWrapper wrapper = new EntityWrapper(paramRelation);
                List<MessageParamRelation> list = messageParamRelationService.selectList(wrapper);
                if (CollectionUtils.isNotEmpty(list)) {
                    for (MessageParamRelation a : list) {
                        messageContext = messageUtil.buildMessageContext(a.getFieldType(), messageContext, taskVo);
                    }
                }
            }
        }
        logger.info("解析完的消息内容为：" + messageContext);
        return messageContext;
    }

    /**
     * 根据可见范围类型获取相应人员集合
     *
     * @param remind
     * @param
     * @return
     */
    public List<AccountVO> getAllAccounts(MessageRemind remind) {


        List<Long> messageIds = new ArrayList<>();

        if (Constans.PUBLIC_SITE.equals(remind.getVisibleRange())) {
            //获取全平台的人
            //如果站点下人数太多会导致查询用户接口超时，故已弃用
//            return queryAccount.getAccountBySiteId(remind.getSiteId());
            //按道理不可能到这一步
            return new ArrayList<>();
        } else {
            if (remind.getMessageType() == 1) {
                //消息类型为自定义
                messageIds.add(remind.getMessageId());
            }
            //获取业务指定范围的人
            return getVisibleAccounts(messageIds, remind.getRelationType(), remind.getRelationId(), remind.getCompanyId());
        }
    }

    /**
     * 根据业务类型获取指定范围相应人员集合
     *
     * @param messageIds
     * @param taskType
     * @param taskId
     * @param companyId
     * @return
     */
    public List<AccountVO> getVisibleAccounts(List<Long> messageIds, Integer taskType, Long taskId, Long companyId) {
        if (CollectionUtils.isNotEmpty(messageIds)) {

            List<MessageVisibleRange> ranges = messageVisibleRangeMapper.getList(messageIds);
            if (CollectionUtils.isNotEmpty(ranges)) {
                return queryAccount.getMessageAccounts(ranges, companyId);
            }
        } else {

            logger.info("taskType= " + taskType + "||taskId:" + taskId + "||companyId:" + companyId);
            //获取各业务的可见范围人员
            if (taskType != null && taskId != null) {
                return queryAccount.getVisibleRangeByTask(taskType, taskId, companyId);
            }
        }
        return new ArrayList<>();
    }


    /**
     * 更新待发消息状态
     *
     * @param messageRemindId
     * @param messageStatus   消息状态
     */
    public void updateMessageRemindStatus(Long messageRemindId, Integer messageStatus) {
        MessageRemind messageRemind = new MessageRemind();
        messageRemind.setStatus(messageStatus);
        messageRemind.setId(messageRemindId);
        try {
            messageRemind.updateById();
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("更新待发消息状态异常");
            throw e;
        }
    }

    /**
     * 更新主体消息状态
     *
     * @param messageId
     * @param messageStatus 消息状态
     */
    public void updateMessageStatus(Long messageId, Integer messageStatus) {
        Message message = new Message();
        message.setId(messageId);
        message.setStatus(messageStatus);
        try {
            message.updateById();
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("更新主体消息状态异常");
            throw e;
        }
    }

    /**
     * 开始给用户发消息
     *
     * @param accountVOS    可见范围用户
     * @param messageRemind 待发消息对象
     */
    public Integer insertMessageAccount(List<AccountVO> accountVOS, MessageRemind messageRemind) {

        if (CollectionUtils.isNotEmpty(accountVOS)) {
            List<MessageAccount> messageAccounts = new ArrayList<>(accountVOS.size());
            for (AccountVO a : accountVOS) {
                MessageAccount messageAccount = new MessageAccount();
                messageAccount.setId(idGenerator.generate());
                messageAccount.setDeleted(Constans.NOT_DELETED);
                messageAccount.setMessageId(messageRemind.getMessageId());
                messageAccount.setMessageRemindId(messageRemind.getId());
                messageAccount.setVisibleRange(messageRemind.getVisibleRange());
                messageAccount.setSiteId(messageRemind.getSiteId());
                messageAccount.setCompanyId(messageRemind.getCompanyId());
                messageAccount.setCreateById(messageRemind.getCreateById());
                messageAccount.setCreateByName(messageRemind.getCreateByName());
                messageAccount.setCreateTime(messageRemind.getSendTime());
                messageAccount.setReaded(Constans.NOT_READ);
                messageAccount.setAccountId(a.getId());
                //修改为在查询的时候对用户名参数进行赋值
//                //  这里只是为了把用户名称给赋值,业务参数赋值已在业务保存待发消息时已处理
//                String completeContext = messageUtil.getCompleteContext(messageRemind.getMessageContext(), a);
                messageAccount.setContext(messageRemind.getMessageContext());
                messageAccount.setRelationId(messageRemind.getRelationId());
                messageAccount.setRelationType(messageRemind.getRelationType());
                messageAccounts.add(messageAccount);
            }
            try {
                if (messageAccountService.insertBatch(messageAccounts)) {
                    return 1;
                }
            } catch (Exception e) {
                e.printStackTrace();
                logger.info("发消息异常");
                throw e;
            }
        } else {
            logger.info("用户为空！！！");
        }
        return -1;
    }

    /**
     * 开始给用户发消息  全平台
     *
     * @param messageRemind 待发消息对象
     */
    public Integer insertMessageAccountByPublic(MessageRemind messageRemind) {

        MessageAccount messageAccount = new MessageAccount();
        messageAccount.setId(idGenerator.generate());
        messageAccount.setDeleted(Constans.NOT_DELETED);
        messageAccount.setMessageId(messageRemind.getMessageId());
        messageAccount.setMessageRemindId(messageRemind.getId());
        messageAccount.setVisibleRange(messageRemind.getVisibleRange());
        messageAccount.setSiteId(messageRemind.getSiteId());
        messageAccount.setCompanyId(messageRemind.getCompanyId());
        messageAccount.setCreateById(messageRemind.getCreateById());
        messageAccount.setCreateByName(messageRemind.getCreateByName());
        messageAccount.setCreateTime(messageRemind.getSendTime());
        messageAccount.setReaded(Constans.NOT_READ);
        messageAccount.setContext(messageRemind.getMessageContext());
        messageAccount.setRelationId(messageRemind.getRelationId());
        messageAccount.setRelationType(messageRemind.getRelationType());

        try {
            if (messageAccountService.insert(messageAccount)) {
                return 1;
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("发消息异常");
            throw e;
        }
        return -1;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertMessageRemind(com.yizhi.message.application.vo.MessageRemindVo vo) {
        RequestContext context = vo.getRequestContext();
        String messageContext = "";
        Date date = new Date();
        if (vo != null) {
            //先判断是否是修改待发消息的状态
            if (vo.getTaskStatusUpdate()) {
                updateMessageRemindByTaskId(vo);
                return 0;
            }
            //判断是否是调研复制场景
            if (null != vo.getIsCopy() && vo.getIsCopy()) {
                copyMessage(vo);
                return 0;
            }
            //计划同步培训的可见范围
            if (vo.getVisibleRangeUpdate()) {
                updateMessageRemindByTaskId(vo);
                return 0;
            }

            //如果业务关闭了消息提醒，则删除对应待发消息
            if (vo.getHasDeleted()) {
                deleteMessageRemind(vo);
                return 0;
            }
            //如果业务只是修改了，则先删除再添加
            if (vo.getIsChangge()) {
                deleteMessageRemind(vo);
            }

            //自定义消息不需要解析业务参数
            messageContext = vo.getMessageContext();
            if (vo.getMessageType() != 1) {
                //先组装好消息内容
                messageContext = getContext(vo, vo.getTaskVo());
            }

            if (vo.getMessageType() == 2) {
                try {
                    Integer num = 0;
                    if (CollectionUtils.isNotEmpty(vo.getMessageTaskRemindVos())) {
                        List<com.yizhi.message.application.vo.MessageTaskRemindVo> messageTaskRemindVos = vo.getMessageTaskRemindVos();
                        //保存待发消息
                        for (com.yizhi.message.application.vo.MessageTaskRemindVo vo1 : messageTaskRemindVos) {
                            //作业没有开始时间，如果有该数据，则过滤
                            if (vo1.getTimeEventType() == 1 && vo.getRelationType().equals(6)) {
                                continue;
                            }
                            insertMessageRemind(vo, messageContext, context, vo1, date);
                            num++;
                        }
                        if (num == vo.getMessageTaskRemindVos().size()) {
                            return 1;
                        }
                    } else {
                        logger.info("提醒时间设置为空！！！！");
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.info("保存待发消息异常");
                    throw e;
                }

            } else {
                insertMessageRemind(vo, messageContext, context, null, date);
            }

        }
        return -1;
    }

    /**
     * 复制调研时需要
     *
     * @param vo
     * @return
     */
    public Integer copyMessage(com.yizhi.message.application.vo.MessageRemindVo vo) {
        if (vo.getRelationType() == null || vo.getRelationId() == null || vo.getOldRelationId() == null) {
            logger.info("copy待发消息时，缺少必要参数");
            return -1;
        }
        MessageRemind remind = new MessageRemind();
        RequestContext requestContext = vo.getRequestContext();
        remind.setRelationType(vo.getRelationType());
        remind.setRelationId(vo.getOldRelationId());
        EntityWrapper<MessageRemind> wrapper = new EntityWrapper(remind);
        List<MessageRemind> messageReminds = messageRemindMapper.selectList(wrapper);
        if (CollectionUtils.isNotEmpty(messageReminds)) {
            messageReminds.forEach(a -> {
                a.setId(idGenerator.generate());
                a.setRelationId(vo.getRelationId());
                a.setCreateTime(new Date());
                a.setCreateByName(requestContext.getAccountName());
                a.setCreateById(requestContext.getAccountId());
                a.setUpdateByName(null);
                a.setUpdateById(null);
                a.insert();
            });
        }
        return 1;
    }

    /**
     * 修改待发消息业务状态根据业务id
     *
     * @param vo
     * @return
     */
    public Integer updateMessageRemindByTaskId(com.yizhi.message.application.vo.MessageRemindVo vo) {
        if (vo.getRelationType() == null || vo.getRelationId() == null) {
            logger.info("修改待发消息时，缺少必要参数");
            return -1;
        }
        RequestContext requestContext = vo.getRequestContext();
        MessageRemind messageRemind = new MessageRemind();
        if (vo.getTaskStatusUpdate()) {
            messageRemind.setTaskStatus(vo.getTaskStatus());
            logger.info("修改待发消息业务状态成功！！！！");
        }
        if (vo.getVisibleRangeUpdate()) {
            messageRemind.setVisibleRange(messageUtil.getVisibleRange(vo.getRelationType(), vo.getVisibleRange()));
            logger.info("修改待发消息可见范围成功！！！！");
        }
        messageRemind.setUpdateById(requestContext.getAccountId());
        messageRemind.setUpdateTime(new Date());
        messageRemind.setUpdateByName(requestContext.getAccountName());
        EntityWrapper wrapper = new EntityWrapper();
        wrapper.eq("deleted", 0)
                .ne("status", 4)
                .eq("relation_type", vo.getRelationType())
                .eq("relation_id", vo.getRelationId())
                .eq("message_type", 2);


        return messageRemindMapper.update(messageRemind, wrapper);

    }

    /**
     * 保存待发消息
     *
     * @param vo
     * @param messageContext
     * @param context
     * @param vo1
     * @param date
     * @return
     */
    public Integer insertMessageRemind(com.yizhi.message.application.vo.MessageRemindVo vo, String messageContext, RequestContext context, com.yizhi.message.application.vo.MessageTaskRemindVo vo1, Date date) {
        MessageRemind messageRemind = new MessageRemind();
        Long messageRemindId = idGenerator.generate();
        BeanUtils.copyProperties(vo, messageRemind);
        messageRemind.setId(messageRemindId);
        messageRemind.setMessageContext(messageContext);
        messageRemind.setSiteId(context.getSiteId());
        messageRemind.setStatus(MessageStatus.TO_SEND.getCode());
        messageRemind.setCompanyId(context.getCompanyId());
        messageRemind.setCreateById(context.getAccountId());
        messageRemind.setCreateByName(context.getAccountName());
        messageRemind.setCreateTime(date);
        if (vo.getMessageType() == 2) {
            messageRemind.setTaskStatus(vo.getTaskStatus());
        }
        //统一可见范围类型
        if (vo.getMessageType() == 1) {
            messageRemind.setVisibleRange(vo.getVisibleRange());
        } else {
            messageRemind.setVisibleRange(messageUtil.getVisibleRange(vo.getRelationType(), vo.getVisibleRange()));
        }
        if (vo1 != null) {
            messageRemind.setTimeEventType(vo1.getTimeEventType());
            messageRemind.setTimeType(vo1.getTimeType());
            if (vo1.getTimeEventType() != 3) {
                //计算最终发送时间
                Date sendTime = getSendTime(vo1.getTimeEventType(), vo1.getTimeType(), vo.getTaskVo().getTaskStratTime(), vo.getTaskVo().getTaskEndTime());
                messageRemind.setSendTime(sendTime);
            } else {
                messageRemind.setSendTime(vo1.getSendTime());
            }
        } else {
            messageRemind.setSendTime(vo.getSendTime());
        }
        try {
            messageRemind.insert();
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("保存待发消息异常");
            throw e;
        }
        return 0;
    }


    /**
     * 计算最终的发送时间
     *
     * @param timeEvenType  1:业务开始时间  2：业务结束时间  3：自定义时间
     * @param timeType      详情请看TimeType类
     * @param taskStartTime 业务开始时间
     * @param taskEndTime   业务结束时间
     * @return
     */
    public static Date getSendTime(Integer timeEvenType, Integer timeType, Date taskStartTime, Date taskEndTime) {
        //只有不为自定义才需要计算发送时间
        logger.info("获取到的参数为：" + "timeEvenType:" + timeEvenType + "||timeType:" + timeType
                + "||taskStartTime:" + taskStartTime
                + "||taskEndTime:" + taskEndTime);
        Date sendTime = null;
        if (timeEvenType == 1 && taskStartTime != null) {
            sendTime = DateConvertor.calSendTime(taskStartTime, timeType);

        } else if (timeEvenType == 2 && taskEndTime != null) {
            sendTime = DateConvertor.calSendTime(taskEndTime, timeType);
        }
        logger.info("解析后发送时间为：" + sendTime);
        return sendTime;
    }

    public static void main(String[] args) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        long time = System.currentTimeMillis();//获取当前系统时间
        Date date = new Date();
        date.setTime(time);
        System.out.println(format.format(getSendTime(1, 5, date, null)));
        ;
    }

    @Override
    public MessageVo get(Integer relationType, Long relationId) {
        MessageVo messageVo = null;

        EntityWrapper wrapper = new EntityWrapper();
        wrapper.eq("relation_type", relationType);
        wrapper.eq("relation_id", relationId);
        wrapper.eq("site_id", ContextHolder.get().getSiteId());
        wrapper.eq("deleted", Constans.NOT_DELETED);
        List<MessageRemind> list = messageRemindMapper.selectList(wrapper);
        if (CollectionUtils.isNotEmpty(list)) {
            //获取主体消息id、发送方式
            MessageRemind messageRemind = list.get(0);
            messageVo = new MessageVo();

            //获取模板名称
            Message message = messageMapper.selectById(messageRemind.getMessageId());
            MessageContext messageContext = new MessageContext();
            messageContext.setMessageId(messageRemind.getMessageId());
            //获取原始消息内容
            messageContext = messageContextMapper.selectOne(messageContext);

            messageVo.setSendType(messageRemind.getSendType());
            messageVo.setTemplateName(message.getTemplateName());
            messageVo.setContext(messageContext.getContext());
            messageVo.setId(message.getId());

            for (MessageRemind remind : list) {
                com.yizhi.message.application.vo.MessageTaskRemindVo taskRemindVo = new com.yizhi.message.application.vo.MessageTaskRemindVo();
                taskRemindVo.setSendTime(remind.getSendTime());
                taskRemindVo.setTimeEventType(remind.getTimeEventType());
                taskRemindVo.setTimeType(remind.getTimeType());
                taskRemindVo.setMessageRemindId(remind.getId());
                messageVo.getMessageTaskRemindVos().add(taskRemindVo);
            }
        }
        return messageVo;
    }

    /**
     * 删除对应待发消息
     *
     * @param messageRemindVo
     */
    public void deleteMessageRemind(com.yizhi.message.application.vo.MessageRemindVo messageRemindVo) {

        if (messageRemindVo.getMessageType() != 1 && (messageRemindVo.getRelationId() == null || messageRemindVo.getRelationType() == null)) {
            logger.info("删除待发消息需要的参数缺少！");
            return;
        }

        RequestContext requestContext = messageRemindVo.getRequestContext();

        EntityWrapper wrapper = new EntityWrapper();
        wrapper.eq("deleted", Constans.NOT_DELETED);

        if (messageRemindVo.getMessageType() != 1 && messageRemindVo.getRelationId() != null && messageRemindVo.getRelationType() != null) {
            wrapper.eq("relation_type", messageRemindVo.getRelationType());
            wrapper.eq("relation_id", messageRemindVo.getRelationId());

        } else {
            wrapper.eq("message_id", messageRemindVo.getMessageId());
        }
        MessageRemind messageRemind = new MessageRemind();
        messageRemind.setDeleted(Constans.DELETED);
        if (null != requestContext) {
            messageRemind.setUpdateById(requestContext.getAccountId());
            messageRemind.setUpdateTime(new Date());
            messageRemind.setUpdateByName(requestContext.getAccountName());
        }
        messageRemindMapper.update(messageRemind, wrapper);

        logger.info("删除或修改待发消息状态成功！！！！");
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateMessageRemind(com.yizhi.message.application.vo.MessageRemindVo messageRemindVo) {
        RequestContext context = ContextHolder.get();
        Date date = new Date();

        try {
            //业务关闭了提醒
            if (messageRemindVo.getIsChangge() == true) {

                EntityWrapper wrapper = new EntityWrapper();
                wrapper.eq("relation_type", messageRemindVo.getRelationType());
                wrapper.eq("relation_id", messageRemindVo.getRelationId());
                wrapper.eq("deleted", Constans.NOT_DELETED);
                List<MessageRemind> list = messageRemindMapper.selectList(wrapper);
                if (CollectionUtils.isNotEmpty(list)) {
                    //先删除该业务的待发消息
                    MessageRemind messageRemind = new MessageRemind();
                    messageRemind.setDeleted(Constans.DELETED);
                    if (messageRemindMapper.update(messageRemind, wrapper) < 1) {
                        logger.info("删除业务待发消息失败");
                    }
                }
                try {
                    return insertMessageRemind(messageRemindVo);
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.info("保存待发消息异常");
                    throw e;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("业务提醒修改异常");
            throw e;
        }
        return -1;
    }
}
